@@ -280,16 +280,16 @@ candl_program_deps_to_string(CandlDependence* dependence)
 	  /* Output dependence information. */
 	  sprintf(buff, "# Description of dependence %d\n"
 		  "# type\n%s\n# From statement id\n%d\n"
-		  "# To statement id\n%d\n# Array id\n%d\n"
+		  "# To statement id\n%d\n# Depth \n%d\n# Array id\n%d\n"
 		  "# Dependence domain\n%d %d\n", nb_deps, type,
-		  tmp->source->label, tmp->target->label,
+		  tmp->source->label, tmp->target->label, tmp->depth,
 		  refs, tmp->domain->NbRows, tmp->domain->NbColumns);
 	  strcat(buffer, buff);
 	  /* Output dependence domain. */
 	  pbuffer = buffer + strlen(buffer);
 	  for (i = 0; i < tmp->domain->NbRows; ++i)
 	    {
-	      for (j = 1; j < tmp->domain->NbColumns; ++j)
+	      for (j = 0; j < tmp->domain->NbColumns; ++j)
 		{
 		  sprintf(buff, "%lld ", CANDL_get_si(tmp->domain->p[i][j]));
 		  szbuff = strlen(buff);
@@ -328,8 +328,229 @@ candl_program_deps_to_string(CandlDependence* dependence)
  *
  */
 #ifdef CANDL_SUPPORTS_SCOPLIB
-void candl_dependence_print_scop(FILE* input, FILE* output,
-				 CandlDependence* dependence)
+/**
+ * Read one dependence from a string.
+ *
+ */
+static
+CandlDependence* candl_dependence_read_one_dep(char* str, char** next,
+					       CandlProgram* program)
+{
+  CandlDependence* dep = candl_dependence_malloc();
+  CandlMatrix* msource = NULL;
+  CandlMatrix* mtarget = NULL;
+  char buffer[1024];
+
+  int i, j, k;
+  int id;
+  int id2;
+  /* # Description of dependence xxx */
+  for (; *str != '\n'; ++str);
+  ++str;
+
+  /* # type */
+  for (; *str != '\n'; ++str);
+  ++str;
+
+  /* {RAW,RAR,WAR,WAW} #(type) */
+  for (i = 0; *str != ' '; ++str, ++i)
+    buffer[i] = *str;
+  buffer[i] = '\0';
+  if (! strcmp(buffer, "RAW"))
+    dep->type = CANDL_RAW;
+  else if (! strcmp(buffer, "RAR"))
+    dep->type = CANDL_RAR;
+  else if (! strcmp(buffer, "WAR"))
+    dep->type = CANDL_WAR;
+  else if (! strcmp(buffer, "WAW"))
+    dep->type = CANDL_WAW;
+  for (; *str != '\n'; ++str);
+  ++str;
+
+  /* # From statement xxx */
+  for (; *str != '\n'; ++str);
+  ++str;
+  /* stmt-id */
+  for (i = 0; *str != '\n'; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  id = atoi(buffer);
+  for (i = 0; i < program->nb_statements; ++i)
+    if (program->statement[i]->label == id)
+      {
+	dep->source = program->statement[i];
+	break;
+      }
+
+  /* # To statement xxx */
+  for (; *str != '\n'; ++str);
+  ++str;
+  /* stmt-id */
+  for (i = 0; *str != '\n'; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  id = atoi(buffer);
+  for (i = 0; i < program->nb_statements; ++i)
+    if (program->statement[i]->label == id)
+      {
+	dep->target = program->statement[i];
+	break;
+      }
+
+  /* # Depth */
+  for (; *str != '\n'; ++str);
+  ++str;
+  /* depth */
+  for (i = 0; *str != '\n'; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  dep->depth = atoi(buffer);
+
+  /* # Array id */
+  for (; *str != '\n'; ++str);
+  ++str;
+  /* array-id */
+  for (i = 0; *str != '\n'; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  id = atoi(buffer);
+  switch (dep->type)
+    {
+    case CANDL_RAW:
+      msource = dep->source->written;
+      mtarget = dep->target->read;
+      break;
+    case CANDL_WAR:
+      msource = dep->source->read;
+      mtarget = dep->target->written;
+      break;
+    case CANDL_WAW:
+      msource = dep->source->written;
+      mtarget = dep->target->written;
+      break;
+    case CANDL_RAR:
+      msource = dep->source->read;
+      mtarget = dep->target->read;
+      break;
+    default:
+      exit(1);
+      break;
+    }
+  for (i = 0; i < msource->NbRows && msource->p[i][0] != id; ++i)
+    ;
+  if (i < msource->NbRows)
+    dep->ref_source = i;
+  for (i = 0; i < mtarget->NbRows && mtarget->p[i][0] != id; ++i)
+    ;
+  if (i < mtarget->NbRows)
+    dep->ref_target = i;
+
+  /* # Dependence domain */
+  for (; *str != '\n'; ++str);
+  ++str;
+
+  /* nb-row nb-col */
+  for (i = 0; *str != ' '; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  id = atoi(buffer);
+  for (i = 0; *str != '\n'; ++str, ++i)
+    buffer[i] = *str;
+  ++str;
+  buffer[i] = '\0';
+  id2 = atoi(buffer);
+
+  dep->domain = candl_matrix_malloc(id, id2);
+  /* Read matrix elements. */
+  for (j = 0; j < id; ++j)
+    {
+    for (k = 0; k < id2; ++k)
+      {
+	while (*str && *str == ' ')
+	  str++;
+	for (i = 0; *str != '\n' && *str != ' '; ++str, ++i)
+	  buffer[i] = *str;
+	buffer[i] = '\0';
+	++str;
+	CANDL_set_si(dep->domain->p[j][k], atoi(buffer));
+      }
+    if (*(str - 1) != '\n')
+      {
+	for (; *str != '\n'; ++str);
+	++str;
+      }
+    }
+  /* Store the next character in the string to be analyzed. */
+  *next = str;
+
+  return dep;
+}
+
+/**
+ * Retrieve a CandlDependence list from the option tag in the scop.
+ *
+ */
+CandlDependence* candl_dependence_read_from_scop(scoplib_scop_p scop,
+						 CandlProgram* program)
+{
+  CandlDependence* first = NULL;
+  CandlDependence* currdep = NULL;
+
+  char* deps = scoplib_scop_tag_content(scop,
+					"<dependence-polyhedra>",
+					"</dependence-polyhedra>");
+  /* No dependence, nothing to do. */
+  if (deps == NULL)
+    return NULL;
+
+  /* Keep the starting address of the array. */
+  char* base = deps;
+
+  int i;
+  int depcount;
+  /* Get the number of dependences. */
+  char buffer_nb[32];
+  /* # Number of dependences */
+  for (; *deps != '\n'; ++deps);
+  ++deps;
+  for (i = 0; *deps != '\n'; ++i, ++deps)
+    buffer_nb[i] = deps[i];
+  buffer_nb[i] = '\0';
+  ++deps;
+  int nbdeps = atoi (buffer_nb);
+  char* next;
+  /* For each of them, read 1 and shift of the read size. */
+  for (depcount = 0; depcount < nbdeps; ++depcount)
+    {
+      CandlDependence* adep =
+	candl_dependence_read_one_dep(deps, &next, program);
+      if (first == NULL)
+	currdep = first = adep;
+      else
+	{
+	  currdep->next = adep;
+	  currdep = currdep->next;
+	}
+      deps = next;
+    }
+
+  /* Be clean. */
+  free(base);
+
+  return first;
+}
+
+/**
+ * Update the scop option tag with the dependence list.
+ *
+ */
+void candl_dependence_update_scop_with_deps(scoplib_scop_p scop,
+					    CandlDependence* dependence)
 {
   char* start;
   char* stop;
@@ -339,25 +560,19 @@ void candl_dependence_print_scop(FILE* input, FILE* output,
   char* newopttags;
   char* curr;
   char* tmp;
-  scoplib_scop_p scop;
   int size = 0;
   int size_newdeps;
   int size_olddeps = 0;
   int size_optiontags;
 
-  /* Go to the beginning of the file. */
-  rewind(input);
-
-  /* Re-read the options tags. */
-  scop = scoplib_scop_read(input);
   start = stop = scop->optiontags;
   /* Get the candl tag, if any. */
   content = scoplib_scop_tag_content(scop, "<candl>", "</candl>");
   if (content)
     {
       /* Get the dependence tag, if any. */
-      olddeps = scoplib_scop_tag_content_from_string(content, "<dependences>",
-						     "</dependences>");
+      olddeps = scoplib_scop_tag_content_from_string
+	(content, "<dependence-polyhedra>", "</dependence-polyhedra>");
       /* Seek for the correct start/stop characters to insert
 	 dependences. */
       if (olddeps)
@@ -384,10 +599,11 @@ void candl_dependence_print_scop(FILE* input, FILE* output,
   size_newdeps = newdeps ? strlen(newdeps) : 0;
   size_optiontags = scop->optiontags ? strlen(scop->optiontags) : 0;
   if (content == NULL)
-    size = strlen("<candl>") + strlen("</candl>") + strlen("<dependence>")
-      + strlen("</dependence>");
+    size = strlen("<candl>") + strlen("</candl>") +
+      strlen("<dependence-polyhedra>")
+      + strlen("</dependence-polyhedra>");
   else if (olddeps == NULL)
-    size = strlen("<dependence>") + strlen("</dependence>");
+    size = strlen("<dependence-polyhedra>") + strlen("</dependence-polyhedra>");
   else
     size = 0;
   newopttags = (char*) malloc((size_newdeps + size_optiontags
@@ -409,21 +625,22 @@ void candl_dependence_print_scop(FILE* input, FILE* output,
     }
   if (olddeps == NULL)
     {
-      strcat(newopttags, "<dependences>\n");
-      curr += strlen("<dependences>\n");
+      strcat(newopttags, "<dependence-polyhedra>\n");
+      curr += strlen("<dependence-polyhedra>\n");
     }
   /* Copy the program dependences. */
   for (tmp = newdeps; tmp && *tmp; ++tmp)
       *(curr++) = *tmp;
+  *curr = '\0';
   /* Copy the candl tags, if needed. */
   if (olddeps == NULL)
     {
-      strcat(newopttags, "</dependences>\n");
-      curr += strlen("</dependences>\n");
+      strcat(curr, "</dependence-polyhedra>\n");
+      curr += strlen("</dependence-polyhedra>\n");
     }
   if (content == NULL)
     {
-      strcat(newopttags, "</candl>\n");
+      strcat(curr, "</candl>\n");
       curr += strlen("</candl>\n");
     }
   /* Copy the remainder of the options. */
@@ -435,9 +652,6 @@ void candl_dependence_print_scop(FILE* input, FILE* output,
     free(scop->optiontags);
   scop->optiontags = newopttags;
 
-  /* Dump the .scop. */
-  scoplib_scop_print_dot_scop(output, scop);
-
   /* Be clean. */
   if (content)
     free(content);
@@ -446,6 +660,28 @@ void candl_dependence_print_scop(FILE* input, FILE* output,
   if (newdeps)
     free(newdeps);
 }
+
+/**
+ * Print the scop, containing the list of dependences.
+ *
+ */
+void candl_dependence_print_scop(FILE* input, FILE* output,
+				 CandlDependence* dependence)
+{
+  scoplib_scop_p scop;
+
+  /* Go to the beginning of the file. */
+  rewind(input);
+
+  /* Re-read the options tags. */
+  scop = scoplib_scop_read(input);
+
+  /* Embed the dependences in the scop option tag. */
+  candl_dependence_update_scop_with_deps(scop, dependence);
+
+  /* Dump the .scop. */
+  scoplib_scop_print_dot_scop(output, scop);
+}
 #endif
 
 
@@ -1089,7 +1325,6 @@ candl_dependence_p candl_dependence(candl_program_p program,
  * This function returns true if the variable indexed by 'var_index'
  * is a scalar in the whole program.
  */
-static
 int
 candl_dependence_var_is_scalar (candl_program_p program, int var_index)
 {
@@ -1689,7 +1924,7 @@ candl_dependence_is_loop_carried (candl_program_p program,
 {
   int i, j;
 
-  /* Ensure soure and sink share common loop 'loop_index', and that
+  /* Ensure source and sink share common loop 'loop_index', and that
      dependence depth is consistent with the considered loop. */
   for (i = 0; i < dep->source->depth; ++i)
     if (dep->source->index[i] == loop_index)
@@ -1709,6 +1944,10 @@ candl_dependence_is_loop_carried (candl_program_p program,
 				       dep->domain->NbColumns);
   CANDL_set_si(m->p[m->NbRows - 2][i + 1], -1);
   CANDL_set_si(m->p[m->NbRows - 1][dep->source->depth + 1 + j], -1);
+  /* Copy the rest of the matrix. */
+  for (i = 0; i < dep->domain->NbRows; ++i)
+    for (j = 0; j < dep->domain->NbColumns; ++j)
+      CANDL_assign(m->p[i][j], dep->domain->p[i][j]);
   /* Compute real lb of loops. */
   Entier lb; CANDL_init(lb);
   candl_dependence_compute_lb (m, &lb, i + 1);
@@ -1716,12 +1955,11 @@ candl_dependence_is_loop_carried (candl_program_p program,
   candl_dependence_compute_lb (m, &lb, dep->source->depth + 1 + j);
   CANDL_assign(m->p[m->NbRows - 1][m->NbColumns - 1], lb);
   CANDL_clear(lb);
-  /* Copy the rest of the matrix. */
-  for (i = 0; i < dep->domain->NbRows; ++i)
-    for (j = 0; j < dep->domain->NbColumns; ++j)
-      CANDL_assign(m->p[i][j], dep->domain->p[i][j]);
   int ret = candl_matrix_check_point(m, program->context);
 
+  /* Be clean. */
+  candl_matrix_free(m);
+
   return !ret;
 }
 
@@ -2043,107 +2281,109 @@ candl_num_dependences(CandlDependence *candl_deps)
  *
  * num: number of Pip matrices returned
  *
- * TODO: cleanup the arguments and make this function general - free of
- * CandlProgram, CandlDependence arguments  (very easy to do -- too dumb of me
- * to have interfaced it this way)
  **/
-PipMatrix *quast_to_polyhedra (PipQuast *quast, int *num,
-        int nvar, int npar)
+static
+PipMatrix **quast_to_polyhedra (PipQuast *quast, int *num,
+				int nvar, int npar)
 {
     int num1, num2;
-    PipMatrix *ep, *tp, *qp;
+    PipMatrix** ep;
+    PipMatrix** tp;
+    PipMatrix** qp;
     int i, j;
-
-    if (quast == NULL)  {
+    if (quast == NULL)
+      {
         *num = 0;
         return NULL;
-    }
-
-    if (quast->condition != NULL)   {
+      }
 
+    if (quast->condition != NULL)
+      {
         tp = quast_to_polyhedra(quast->next_then, &num1, nvar, npar);
         ep = quast_to_polyhedra(quast->next_else, &num2, nvar, npar);
 
         /* Each of the matrices in the then tree needs to be augmented with
          * the condition */
-        for (i=0; i<num1; i++)  {
-            int nrows = tp[i].NbRows;
-            CANDL_set_si(tp[i].p[nrows][0], 1);
-            for (j=1; j<1+nvar; j++) {
-                CANDL_set_si(tp[i].p[nrows][j], 0);
-            }
-            for (j=0; j<npar+1; j++)  {
-                CANDL_assign(tp[i].p[nrows][1+nvar+j], quast->condition->the_vector[j]);
-            }
-            tp[i].NbRows++;
-        }
+        for (i = 0; i < num1; i++)
+	  {
+	    int nrows = tp[i]->NbRows;
+	    CANDL_set_si(tp[i]->p[nrows][0], 1);
+	    for (j = 1; j < 1 + nvar; j++)
+	      CANDL_set_si(tp[i]->p[nrows][j], 0);
+	    for (j = 0; j < npar + 1; j++)
+	      CANDL_assign(tp[i]->p[nrows][1+nvar+j],
+			   quast->condition->the_vector[j]);
+	    (tp[i]->NbRows)++;
+	  }
 
-        for (i=0; i<num2; i++)  {
-            int nrows = ep[i].NbRows;
-            /* Inequality */
-            CANDL_set_si(ep[i].p[nrows][0], 1);
-            for (j=1; j<1+nvar; j++) {
-                CANDL_set_si(ep[i].p[nrows][j], 0);
-            }
-            for (j=0; j<npar+1; j++)  {
-                CANDL_set_si(ep[i].p[nrows][1+nvar+j], -CANDL_get_si(quast->condition->the_vector[j]));
-                // ep[i].p[nrows][1+nvar+j] = quast->condition->the_vector[j];
-            }
-            ep[i].NbRows++;
-        }
+        for (i = 0; i < num2; i++)
+	  {
+	    int nrows = ep[i]->NbRows;
+	    /* Inequality */
+	    CANDL_set_si(ep[i]->p[nrows][0], 1);
+	    for (j = 1; j < 1 + nvar; j++)
+	      CANDL_set_si(ep[i]->p[nrows][j], 0);
+	    for (j = 0; j < npar + 1; j++)
+	      CANDL_set_si(ep[i]->p[nrows][1+nvar+j],
+			   -CANDL_get_si(quast->condition->the_vector[j]));
+	    (ep[i]->NbRows)++;
+	  }
 
-        qp = (PipMatrix *) malloc((num1+num2)*sizeof(PipMatrix));
-
-        memcpy(qp, tp, sizeof(PipMatrix)*num1);
-        memcpy(qp+num1, ep, sizeof(PipMatrix)*num2);
+        qp = (PipMatrix **) malloc((num1 + num2) * sizeof(PipMatrix*));
+	for (i = 0; i < num1; ++i)
+	  qp[i] = tp[i];
+	for (i = 0; i < num2; ++i)
+	  qp[i + num1] = ep[i];
 
         *num = num1 + num2;
 
         return qp;
 
-    }else{
+      }
+    else
+      {
         /* quast condition is NULL */
-
-        PipMatrix *lwmatrix = pip_matrix_alloc(nvar+npar, nvar+npar+2);
+        PipMatrix *lwmatrix = pip_matrix_alloc(nvar+npar+1, nvar+npar+2);
 
         PipList *vecList = quast->list;
 
         int count=0;
         while (vecList != NULL) {
-            /* Equality */
-            CANDL_set_si(lwmatrix->p[count][0], 0);
-            for (j=0; j<nvar; j++)   {
-                if (j == count) {
-                    CANDL_set_si(lwmatrix->p[count][j+1], 1);
-                }else{
-                    CANDL_set_si(lwmatrix->p[count][j+1], 0);
-                }
-            }
+	  /* Equality */
+	  CANDL_set_si(lwmatrix->p[count][0], 0);
+	  for (j=0; j<nvar; j++)
+	    if (j == count)
+	      CANDL_set_si(lwmatrix->p[count][j+1], 1);
+	    else
+	      CANDL_set_si(lwmatrix->p[count][j+1], 0);
 
-            for (j=0; j<npar; j++)   {
-                CANDL_set_si(lwmatrix->p[count][j+1+nvar], -CANDL_get_si(vecList->vector->the_vector[j]));
-            }
-            /* Constant portion */
-            if (quast->newparm != NULL) {
-                /* Don't handle newparm for now */
-                CANDL_set_si(lwmatrix->p[count][npar+1+nvar], -CANDL_get_si(vecList->vector->the_vector[npar+1]));
-            }else{
-                CANDL_set_si(lwmatrix->p[count][npar+1+nvar], -CANDL_get_si(vecList->vector->the_vector[npar]));
-            }
+	  for (j=0; j<npar; j++)
+	    CANDL_set_si(lwmatrix->p[count][j+1+nvar],
+			 -CANDL_get_si(vecList->vector->the_vector[j]));
+	  /* Constant portion */
+	  if (quast->newparm != NULL)
+	    /* Don't handle newparm for now */
+	    CANDL_set_si(lwmatrix->p[count][npar+1+nvar],
+			 -CANDL_get_si(vecList->vector->the_vector[npar+1]));
+	  else
+	    CANDL_set_si(lwmatrix->p[count][npar+1+nvar],
+			 -CANDL_get_si(vecList->vector->the_vector[npar]));
 
-            count++;
+	  count++;
 
-            vecList = vecList->next;
+	  vecList = vecList->next;
         }
         lwmatrix->NbRows = count;
 
-        if (count > 0) *num = 1;
-        else *num = 0;
+        if (count > 0)
+	  *num = 1;
+        else
+	  *num = 0;
 
-        // pip_matrix_print(stdout, lwmatrix);
-
-        return lwmatrix;
-    }
+	PipMatrix** ret = (PipMatrix**) malloc(sizeof(PipMatrix*));
+	ret[0] = lwmatrix;
+        return ret;
+      }
 }
 
 
@@ -2154,7 +2394,8 @@ PipMatrix *quast_to_polyhedra (PipQuast *quast, int *num,
  * Returns 0 if lastwriter is computed successfully and dep domain updated,
  * returns 1 otherwise
  */
-int candl_dep_compute_lastwriter (CandlDependence *dep, CandlProgram *prog)
+static
+int candl_dep_compute_lastwriter (CandlDependence **dep, CandlProgram *prog)
 {
     PipQuast *lexmax;
     PipMatrix *new_domain;
@@ -2175,81 +2416,93 @@ int candl_dep_compute_lastwriter (CandlDependence *dep, CandlProgram *prog)
 
     /* Build a context with equalities /inequalities only on the target
      * variables */
-    PipMatrix *context = pip_matrix_alloc(dep->domain->NbRows,
-            dep->target->depth + npar + 2);
+    PipMatrix *context = pip_matrix_alloc((*dep)->domain->NbRows,
+					  (*dep)->target->depth + npar + 2);
 
     int nrows = 0;
-    for (i=0; i<dep->domain->NbRows; i++)  {
-        for (j=1; j<dep->source->depth+1; j++)  {
-            if (dep->domain->p[i][j] != 0) {
-                break;
-            }
-        }
-        if (j == dep->source->depth+1)  {
-            /* Include this in the context */
-            CANDL_assign(context->p[nrows][0], dep->domain->p[i][0]);
-            for (j=1; j < 1+dep->target->depth+npar+1; j++)  {
-                CANDL_assign(context->p[nrows][j], dep->domain->p[i][dep->source->depth+j]);
-            }
-            nrows++;
-        }
-    }
+    for (i = 0; i < (*dep)->domain->NbRows; i++)
+      {
+	for (j = 1; j < (*dep)->source->depth+1; j++)
+	  {
+	    if ((*dep)->domain->p[i][j] != 0)
+	      break;
+	  }
+	if (j == (*dep)->source->depth+1)
+	  {
+	    /* Include this in the context */
+	    CANDL_assign(context->p[nrows][0], (*dep)->domain->p[i][0]);
+	    for (j = 1; j < 1 + (*dep)->target->depth + npar + 1; j++)
+	      CANDL_assign(context->p[nrows][j],
+			   (*dep)->domain->p[i][(*dep)->source->depth+j]);
+
+	    nrows++;
+	  }
+      }
     context->NbRows = nrows;
 
-    // pip_matrix_print(stdout, context);
-
     /* Parameteric lexmax */
-    lexmax = pip_solve(dep->domain, context, -1, pipOptions);
+    lexmax = pip_solve((*dep)->domain, context, -1, pipOptions);
 
     pip_options_free(pipOptions);
 
-    if (lexmax == NULL) {
+    if (lexmax == NULL)
+      {
         printf("WARNING: last writer failed (mostly invalid dependence): %s\n",
                "bailing out safely without modification");
-        pip_matrix_print(stdout, dep->domain);
+        pip_matrix_print(stdout, (*dep)->domain);
         pip_matrix_print(stdout, context);
         return 1;
-    }
-
-    // pip_quast_print(stdout, lexmax, 0);
+      }
 
     int num;
-
-    PipMatrix *qp = quast_to_polyhedra(lexmax, &num, dep->source->depth,
-            dep->target->depth + npar);
-
-    // pip_matrix_print(stdout, qp);
-
-    /* Multiple cases are not handled */
-    if (num >= 2)   {
-        printf("WARNING: last writer failed (incomplete handling): %s\n",
-               "bailing out safely without modification");
-        return 1;
-    }
+    PipMatrix **qp = quast_to_polyhedra(lexmax, &num, (*dep)->source->depth,
+					(*dep)->target->depth + npar);
 
     /* Update the dependence domains */
-    if (num > 0) {
-        /* Just using qp[0] */
+    if (num > 0)
+      {
+	int it_mat;
+	PipMatrix* original_domain = (*dep)->domain;
+	for (it_mat = 0; it_mat < num; ++it_mat)
+	  {
+	    new_domain = pip_matrix_alloc(original_domain->NbRows +
+					  qp[it_mat]->NbRows,
+					  original_domain->NbColumns);
+	    for (i = 0; i < original_domain->NbRows; i++)
+	      for (j = 0; j < original_domain->NbColumns; j++)
+		CANDL_assign(new_domain->p[i][j], original_domain->p[i][j]);
 
-        new_domain = pip_matrix_alloc(dep->domain->NbRows + qp->NbRows,
-                dep->domain->NbColumns);
-        for (i=0; i<dep->domain->NbRows; i++)  {
-            for (j=0; j<dep->domain->NbColumns; j++)  {
-                CANDL_assign(new_domain->p[i][j], dep->domain->p[i][j]);
-            }
-        }
+	    for (i = 0; i < qp[it_mat]->NbRows; i++)
+	      for (j = 0; j < original_domain->NbColumns; j++)
+		CANDL_assign(new_domain->p[i+original_domain->NbRows][j],
+			     qp[it_mat]->p[i][j]);
 
-        for (i=0; i<qp->NbRows; i++)  {
-            for (j=0; j<dep->domain->NbColumns; j++)  {
-                CANDL_assign(new_domain->p[i+dep->domain->NbRows][j], qp->p[i][j]);
-            }
-        }
+	    (*dep)->domain = new_domain;
+	    /* More than 1 pipmatrix from the quast, we need to insert
+	       new dependences to have the union of domains. */
+	    if (it_mat < num - 1)
+	      {
+		CandlDependence* newdep = candl_dependence_malloc ();
+		newdep->source = (*dep)->source;
+		newdep->target = (*dep)->target;
+		newdep->depth = (*dep)->depth;
+		newdep->type = (*dep)->type;
+		newdep->ref_source = (*dep)->ref_source;
+		newdep->ref_target = (*dep)->ref_target;
+		newdep->usr = (*dep)->usr;
+		newdep->next = (*dep)->next;
+		(*dep)->next = newdep;
+		*dep = newdep;
+	      }
+	  }
 
-        pip_matrix_free(dep->domain);
-        dep->domain = new_domain;
-    }
+	pip_matrix_free(original_domain);
+	for (i = 0; i < num; ++i)
+	  pip_matrix_free(qp[i]);
+      }
 
-    pip_matrix_free(qp);
+    if (qp)
+      free(qp);
     pip_quast_free(lexmax);
     pip_matrix_free(context);
 
@@ -2264,12 +2517,12 @@ int candl_dep_compute_lastwriter (CandlDependence *dep, CandlProgram *prog)
  */
 void candl_compute_last_writer (CandlDependence *dep, CandlProgram *prog)
 {
-    // int count=0;
+  // int count=0;
     while (dep != NULL)    {
         if (dep->type == CANDL_RAW || dep->type == CANDL_WAW || dep->type == CANDL_RAR)   {
-            // printf("Last writer for dep %d: %d %d\n", count++, dep->source->depth, dep->target->depth);
+	  // printf("Last writer for dep %d: %d %d\n", count++, dep->source->depth, dep->target->depth);
             // candl_matrix_print(stdout, dep->domain);
-            candl_dep_compute_lastwriter(dep, prog);
+            candl_dep_compute_lastwriter(&dep, prog);
             // candl_matrix_print(stdout, dep->domain);
         }
         dep = dep->next;
