@@ -494,11 +494,11 @@ candl_program_p candl_program_convert_scop(scoplib_scop_p scop, int** indices)
 
       statement->domain = (CandlMatrix*) scoplib_matrix_copy(s->domain->elt);
       /* For the moment, we do not parse the statement to extract its type. */
-      statement->type = CANDL_AFFECTATION;
+      statement->type = CANDL_ASSIGNMENT;
       statement->depth = statement->domain->NbColumns - 2 - scop->nb_parameters;
       statement->written = (CandlMatrix*) scoplib_matrix_copy(s->write);
       if (statement->written == NULL)
-	statement->written = 
+	statement->written =
 	  candl_matrix_malloc(0, statement->domain->NbColumns);
       statement->read = (CandlMatrix*) scoplib_matrix_copy(s->read);
       if (statement->read == NULL)
@@ -554,6 +554,8 @@ candl_program_p candl_program_convert_scop(scoplib_scop_p scop, int** indices)
 	      if (last[j] < val)
 		{
 		  last[j] = val;
+		  for (k = j + 1; k < max_loop_depth; ++k)
+		    last[k] = 0;
 		  for (k = j; k < max_loop_depth; ++k)
 		    cur_index[k] = max + (k - j) + 1;
 		  break;
